@@ -134,10 +134,9 @@ impl AdminRpc for AdminRpcImpl {
 pub fn run(ledger_path: &Path, metadata: AdminRpcRequestMetadata) {
     let admin_rpc_path = ledger_path.join("admin.rpc");
 
-    let event_loop = tokio::runtime::Builder::new()
-        .threaded_scheduler()
-        .enable_all()
+    let event_loop = tokio::runtime::Builder::new_multi_thread()
         .thread_name("sol-adminrpc-el")
+        .enable_all()
         .build()
         .unwrap();
 
